/*****************************************************************************
 *
 * FILE:	mod_date.c
 * DESCRIPTION:	Cockatrice: time/date utilities (module)
 * DATE:	Sat, Mar 16 2002
 * UPDATED:	Sat, Aug 18 2007
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/
 * COPYRIGHT:	(c) 2002-2007 졿Kouichi ABE (WALL), All rights reserved.
 * LICENSE:
 *
 *  Copyright (c) 2002-2007 Kouichi ABE (WALL) <kouichi@MysticWALL.COM>,
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   1. Redistributions of source code must retain the above copyright
 *      notice, this list of conditions and the following disclaimer.
 *
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice, this list of conditions and the following disclaimer in the
 *      documentation and/or other materials provided with the distribution.
 *
 *   THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 *   ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 *   THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE
 *   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *   INTERRUPTION)  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 *   THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: mod_date.c,v 1.1 2007/08/19 16:49:15 kouichi Exp $
 *
 *****************************************************************************/

#if	HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#if	HAVE_UNISTD_H
#include <unistd.h>
#endif	/* HAVE_UNISTD_H */
#include <time.h>
#if	TIME_WITH_SYS_TIME
#include <sys/time.h>
#endif	/* TIME_WITH_SYS_TIME */
#include "cockatrice.h"
#include "module.h"
#include "functions.h"

/*****************************************************************************
 *
 *	Macros and structures definition
 *
 *****************************************************************************/
typedef struct {
  char *	today;
  int		year;
  int		month;	/* 1 .. 12 */
  int		day;	/* 1 .. 31 */
  int		week;	/* 0 .. 6 (0:Sun ... 6:Sat) */
} Date;

#if	ENABLE_JP_MESG
#define	JAN_01	"ö"
#define	JAN_CA	"ͤ"
#define	FEB_11	"ǰ"
#define	MAR_VE	"ʬ"
#define	APR_29	"¤"
#define	MAY_03	"ˡǰ"
#define	MAY_04	"ߤɤ"
#define	MAY_05	"ɤ"
#define	JUL_SD	""
#define	SEP_OP	"Ϸ"
#define	SEP_AE	"ʬ"
#define	OCT_PC	"ΰ"
#define	NOV_03	"ʸ"
#define	NOV_23	"ϫդ"
#define	DEC_23	"ŷ"
#else
#define	JAN_01	"New Year's Day"
#define	JAN_CA	"Coming-of-Age Day"
#define	FEB_11	"National Foundation Day"
#define	MAR_VE	"Vernal Equinox Day"
#define	APR_29	"Showa's Day"
#define	MAY_03	"Constitution Day"
#define	MAY_04	"Green's Day"
#define	MAY_05	"Children's Day"
#define	JUL_SD	"Sea's Day"
#define	SEP_OP	"Old People's Day"
#define	SEP_AE	"Autumnal Equinox Day"
#define	OCT_PC	"Physical Culture Day"
#define	NOV_03	"Culture Day"
#define	NOV_23	"Labor Thanksgiving Day"
#define	DEC_23	"The Emperor's Birthday"
#endif	/* ENABLE_JP_MESG */

/*****************************************************************************
 *
 *	Local functions declaration
 *
 *****************************************************************************/
static void		cm_date_init(void);
static char *		cm_date_get_today(void);
static int		cm_date_get_year_of_today(void);
static int		cm_date_get_month_of_today(void);
static int		cm_date_get_day_of_today(void);
static int		cm_date_get_wday_of_today(void);
static bool		cm_date_is_festival(int, int, int);
static const char *	cm_date_get_festival_name(int, int, int);
static bool		cm_date_is_holiday_in_lieu(int, int, int);
static int		cm_date_get_day_of_the_week(int, int, int);
static char *		cm_date_get_time(void);
static bool		cm_date_check(int, int, int);
#ifdef	__GNUC__
__inline
#endif	/* __GNUC__ */
static int	get2ndMonday(int, int);
#ifdef	__GNUC__
__inline
#endif	/* __GNUC__ */
static int	get3rdMonday(int, int);

static const char *	cm_date_get_rokuyou(int, int, int);
static const char *	cm_date_get_24sekki(int, int, int);
static int	cm_date_get_old_calendar(int, int, int, int *);

/*****************************************************************************
 *
 *	Local variables declaration
 *
 *****************************************************************************/
#if	ENABLE_JP_MESG
static char *	wday[] = {"", "", "", "", "", "", ""};
#else
static char *	wday[] = {"Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"};
#endif	/* ENABLE_JP_MESG */
static Date	date;

/*****************************************************************************
 *
 *	Functions definition
 *
 *****************************************************************************/

/*
 * FUNCTION NAME:	cm_date_init
 * DESCRIPTION:		initialize date
 * ARGUMENTS:		none
 * RETURN VALUE:	none
 */
static void
cm_date_init(void)
{
  time_t	now;
  struct tm *	jst;
  
  time(&now);
  jst = localtime(&now);

  date.year	= jst->tm_year + 1900;
  date.month	= jst->tm_mon + 1;
  date.day	= jst->tm_mday;
  date.week	= jst->tm_wday;

#if	ENABLE_JP_MESG
  if (asprintf(&date.today, "%4dǯ%02d%02d%s",
		date.year, date.month, date.day, wday[date.week]) == -1) {
#ifndef	DATE_DEBUG
    cgi_errno = CGIERR_NO_ENOUGH_MEMORY;
#endif	/* DATE_DEBUG */
  }
#else
  if (asprintf(&date.today, "%4d/%02d/%02d(%s)",
		date.year, date.month, date.day, wday[date.week]) == -1) {
#ifndef	DATE_DEBUG
    cgi_errno = CGIERR_NO_ENOUGH_MEMORY;
#endif	/* DATE_DEBUG */
  }
#endif	/* ENABLE_JP_MESG */
}

static char *	cm_date_get_today(void)		{ return date.today; }
static int	cm_date_get_year_of_today(void)	{ return date.year; }
static int	cm_date_get_month_of_today(void){ return date.month; }
static int	cm_date_get_day_of_today(void)	{ return date.day; }
static int	cm_date_get_wday_of_today(void)	{ return date.week; }


/*
 * FUNCTION NAME:	cm_date_is_festival
 * DESCRIPTION:		Is a day festival?
 * ARGUMENTS:		year, month, day
 * RETURN VALUE:	Festival: true, otherwise: false
 */
static bool
cm_date_is_festival(year, month, day)
	int	year;
	int	month;
	int	day;
{
  int	CoA;	/* Coming-of-Age Day (in January) */
  int	VE;	/* Vernal Equinox Day (1996 - ) (in March) */
  int	AE;	/* Autumnal Equinox Day (1996 - ) (in September) */
  int	SD;	/* Sea's Day (in July) */
  int	OP;	/* Old People's Day (in September) */
  int	PC;	/* Physical Culture Day (in October) */

  CoA	= get2ndMonday(year, 1);
  VE	= (int)(20.712662 + 0.242199 * (double)(year - 1996))
	- (int)((year - 1996) / 4);
  AE	= (int)(23.130023 + 0.242199 * (double)(year - 1996))
  	- (int)((year - 1996) / 4);
  SD	= get3rdMonday(year, 7);
  OP	= get3rdMonday(year, 9);
  PC	= get2ndMonday(year, 10);

  if ((month ==  1 && day ==  1) ||	/* New Year's Day */
      (month ==  1 && day == CoA) ||	/* Coming-of-Age Day */
      (month ==  2 && day == 11) ||	/* National Foundation Day */
      (month ==  3 && day == VE) ||	/* Vernal Equinox Day */
      (month ==  4 && day == 29) ||	/* Green's Day */
      (month ==  5 && day ==  3) ||	/* Constitution Day */
      (month ==  5 && day ==  4) ||	/* National Holiday */
      (month ==  5 && day ==  5) ||	/* Children's Day */
      (month ==  7 && day == SD) ||	/* Sea's Day */
      (month ==  9 && day == OP) ||	/* Old People's Day */
      (month ==  9 && day == AE) ||	/* Autumnal Equinox Day */
      (month == 10 && day == PC) ||	/* Physical Culture Day */
      (month == 11 && day ==  3) ||	/* Culture Day */
      (month == 11 && day == 23) ||	/* Labor Thanksgiving Day */
      (month == 12 && day == 23)) {	/* The Emperor's Birthday */
    return true;
  }

  return false;
}

/*
 * FUNCTION NAME:	cm_date_get_festival_name
 * DESCRIPTION:		get a festival name
 * ARGUMENTS:		year, month, day
 * RETURN VALUE:	Festival: name, otherwise: NULL
 */
static const char *
cm_date_get_festival_name(year, month, day)
	int	year;
	int	month;
	int	day;
{
  int	CoA;	/* Coming-of-Age Day (in January) */
  int	VE;	/* Vernal Equinox Day (1996 - ) (in March) */
  int	AE;	/* Autumnal Equinox Day (1996 - ) (in September) */
  int	SD;	/* Sea's Day (in July) */
  int	OP;	/* Old People's Day (in September) */
  int	PC;	/* Physical Culture Day (in October) */

  CoA	= get2ndMonday(year, 1);
  VE	= (int)(20.712662 + 0.242199 * (double)(year - 1996))
	- (int)((year - 1996) / 4);
  AE	= (int)(23.130023 + 0.242199 * (double)(year - 1996))
  	- (int)((year - 1996) / 4);
  SD	= get3rdMonday(year, 7);
  OP	= get3rdMonday(year, 9);
  PC	= get2ndMonday(year, 10);

  if (month ==  1 && day ==  1)	 return JAN_01;
  if (month ==  1 && day == CoA) return JAN_CA;
  if (month ==  2 && day == 11)	 return FEB_11;
  if (month ==  3 && day == VE)	 return MAR_VE;
  if (month ==  4 && day == 29)	 return APR_29;
  if (month ==  5 && day ==  3)	 return MAY_03;
  if (month ==  5 && day ==  4)	 return MAY_04;
  if (month ==  5 && day ==  5)	 return MAY_05;
  if (month ==  7 && day == SD)	 return JUL_SD;
  if (month ==  9 && day == OP)	 return SEP_OP;
  if (month ==  9 && day == AE)	 return SEP_AE;
  if (month == 10 && day == PC)	 return OCT_PC;
  if (month == 11 && day ==  3)	 return NOV_03;
  if (month == 11 && day == 23)	 return NOV_23;
  if (month == 12 && day == 23)	 return DEC_23;

  return NULL;
}


/*
 * FUNCTION NAME:	cm_date_is_holiday_in_lieu
 * DESCRIPTION:		Is a day holiday in lieu?
 * ARGUMENTS:		year, month, day
 * RETURN VALUE:	Holiday in lieu: true, otherwise: false
 */
static bool
cm_date_is_holiday_in_lieu(year, month, day)
	int	year;
	int	month;
	int	day;
{
#define LEAP(x) ((((x) % 4 == 0) && ((x) % 100 != 0)) || ((x) % 400 == 0))
  static int    mday[] = { 0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };
  int		leap = LEAP(year);
  int		y;
  int		m;
  int		d;

  mday[2] += leap;
  if (day == 1) {
    if (month == 1) {
      m = 12;
      y = year - 1;
    }
    else {
      m = month - 1;
      y = year;
    }
    d = mday[m];
  }
  else {
    d = day - 1;
    m = month;
    y = year;
  }
  mday[2] -= leap;	/* end of revised */

  return (cm_date_is_festival(y, m, d) &&
	  cm_date_get_day_of_the_week(y, m, d) == 0) ? true : false;
}

/*
 * FUNCTION NAME:	get2ndMonday
 * DESCRIPTION:		calculate the 2nd Monday for a Happy Monday
 * ARGUMENTS:		year, month
 * RETURN VALUE:	day of Happy Monday
 */
#ifdef	__GNUC__
__inline
#endif	/* __GNUC__ */
static int
get2ndMonday(year, month)
	int	year;
	int	month;
{
  return 14 - (cm_date_get_day_of_the_week(year, month, 1) + 5) % 7;
}

/*
 * FUNCTION NAME:	get3rdMonday
 * DESCRIPTION:		calculate the 3rd Monday for a Happy Monday
 * ARGUMENTS:		year, month
 * RETURN VALUE:	day of Happy Monday
 */
#ifdef	__GNUC__
__inline
#endif	/* __GNUC__ */
static int
get3rdMonday(year, month)
	int	year;
	int	month;
{
  return 21 - (cm_date_get_day_of_the_week(year, month, 1) + 5) % 7;
}

/*
 * FUNCTION NAME:	cm_date_get_day_of_the_week
 * DESCRIPTION:		calculate a day of the week
 * ARGUMENTS:		year, month, day
 * RETURN VALUE:	Successful: 0:Sunday - 6:Saturday, otherwise: -1
 */
static int
cm_date_get_day_of_the_week(year, month, day)
	int	year;
	int	month;
	int	day;
{
  if (year > 1582) {
    if (month < 3) {
      year--;
      month += 12;
    }
    return (year + (int)(year / 4) - (int)(year / 100) + (int)(year / 400) +
	    (int)((13 * month + 8) / 5) + day) % 7;
  }
  return -1;
}

/*
 * FUNCTION NAME:	cm_date_get_time
 * DESCRIPTION:		return present time
 * ARGUMENTS:		none
 * RETURN VALUE:	successful: time, otherwise: NULL
 */
static char *
cm_date_get_time(void)
{
  time_t	now;
  struct tm *	jst;
  char *	buf;
  
  time(&now);
  jst = localtime(&now);

#if	ENABLE_JP_MESG
  if (asprintf(&buf, "%02d%02dʬ%02d",
		jst->tm_hour, jst->tm_min, jst->tm_sec) == -1) {
    cgi_errno = CGIERR_NO_ENOUGH_MEMORY;
    return NULL;
  }
#else
  if (asprintf(&buf, "%02d:%02d:%02d",
		jst->tm_hour, jst->tm_min, jst->tm_sec) == -1) {
    cgi_errno = CGIERR_NO_ENOUGH_MEMORY;
    return NULL;
  }
#endif	/* ENABLE_JP_MESG */

  return buf;
}

/*
 * FUNCTION NAME:	cm_date_check
 * DESCRIPTION:		check valid date
 * ARGUMENTS:		year, month, day
 * RETURN VALUE:	valid: 1, otherwise: 0
 */
static bool
cm_date_check(year, month, day)
	int	year;
	int	month;
	int	day;
{
  static int	mday[]	= { 0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };
  int		leap	= LEAP(year);
  int		invalid	= 0;

  mday[2] += leap;
  if ( year < 1 ||  year > 3000)	{ invalid++; }
  if (month < 1 || month > 12)		{ invalid++; }
  if (  day < 1 ||   day > mday[month]) { invalid++; }
  mday[2] -= leap;

  return !invalid;
}

/*****************************************************************************/

/*
 * NOTE:
 *	׻ΥꥸʥΥץȤϹ awk ǤǤ
 *	Ǥޤ
 *	http://www.vector.co.jp/soft/dos/personal/se016093.html
 *
 */
#include <math.h>

/*****************************************************************************
 *
 *	Macros and structures definition
 *
 *****************************************************************************/
/*
 * ߼Ψȡʳ٤Ρ٤饸Ѵ뷸
 */
#define	PI	(3.141592653589793238462)
#define	KXX	(PI / 180.0)

#define	HT_OLD_SIZE	(4)	/* size of old calendar array */
#define	HT_OLD_YEAR	(0)
#define	HT_OLD_LEAP	(1)
#define	HT_OLD_MONTH	(2)
#define	HT_OLD_DAY	(3)

#define	HT_SAKU_SIZE	(3)
#define	HT_SAKU_MONTH	(0)
#define	HT_SAKU_LEAP	(1)
#define	HT_SAKU_DATE	(2)

#define	HT_YMDT_SIZE	(6)
#define	HT_YMDT_YEAR	(0)
#define	HT_YMDT_MONTH	(1)
#define	HT_YMDT_DAY	(2)
#define	HT_YMDT_HOUR	(3)
#define	HT_YMDT_MINUTE	(4)
#define	HT_YMDT_SECOND	(5)

/*****************************************************************************
 *
 *	Local functions declaration
 *
 *****************************************************************************/
static const char *	ht_get_rokuyou(int, int);
static void		ht_calc_kyureki(double, int *);
static void		ht_calc_chuki(double, int, double [][2]);
static void		ht_calc_before_nibun(double, double [][2]);
static double		ht_calc_saku(double);
static double		ht_normalization_angle(double);
static double		ht_longitude_sun(double);
static double		ht_longitude_moon(double);
static double		ht_YMDT2JD(int, int, int, int, int, int);
static void		ht_JD2YMDT(double, int *);
static const char *	ht_check_24sekki(int, int, int);

/*****************************************************************************
 *
 *	Functions definition
 *
 *****************************************************************************/

/*
 * ϻ˻дؿ
 * FGALEL  MES(12):01945 δؿ¤ޤ...
 *˷礵󤢤꤬Ȥޤ
 * 
 * ƤӽФ˥åȤѿ
 * month:η  day:
 */
static const char *
ht_get_rokuyou(month, day)
	int	month;
	int	day;
{
  /* ϻˤ */
  static const char *	tbl_rokuyou[] = {
    "辡", "ͧ", "", "ʩ", "", "ָ"
  };
  int	i;

  i  = month + day - 2;
  i %= 6;

  return tbl_rokuyou[i];
}

/*
 * б롢롣
 *
 * ƤӽФ˥åȤѿ
 * tm0 : ׻աʥꥦ
 * kyureki : γǼ򤫤
 *           kyureki[0] : ǯ
 *           kyureki[1] : ʿ flag .... ʿ:0 :1
 *           kyureki[2] : 
 *           kyureki[3] : 
 *
 */
static void
ht_calc_kyureki(tm0, kyureki)
	double	tm0;
	int *	kyureki;
{
  double	saku[5];
  double	chu[4][2];
  int		lap;
  int		m[5][HT_SAKU_SIZE];
  int		state;
  int		ymdt[HT_YMDT_SIZE];
  register int	i;

  /*
   * ׻оݤľˤʬλ
   * chu[0][0]:ʬλ  chu[0][1]:λ۲
   */
  ht_calc_before_nibun(tm0, chu);

  /*
   * 浤λ׻ʣ׻
   * chu[i][0]:浤λ  chu[i][1]:۲
   */
  for (i = 1; i < 4; i++) {
    ht_calc_chuki(chu[i-1][0] + 32.0, i, chu);
  }

  /*
   * ׻оݤľˤʬľκλ
   */
  saku[0] = ht_calc_saku(chu[0][0]);

  /*
   * λ
   */
  for (i = 1; i < 5; i++) {
    double	tm;

    tm  = saku[i-1];
    tm += 30.0;
    saku[i] = ht_calc_saku(tm);
    /*
     * Ʊ׻ξԤκ26ˤˤϡ
     * ͤ +33ˤƺƼ¹Ԥ롣
     */
    if (fabs(saku[i-1] - saku[i]) <= 26.0) {
      saku[i] = ht_calc_saku(saku[i-1] + 35.0);
    }
  }

  /*
   * saku[1]ʬλˤʤäƤޤäˤϡ
   * 򤵤Τܤ᤮ȹͤơλ򷫤겼ƽ롣
   * κݡ׻saku[4]ˤˤʤäƤʬ䤦ᡢ
   * λ׻롣
   * ʶ̲ζդǺ󤬤ȵ褦...
   */
  if (saku[1] <= chu[0][0]) {
    for (i = 0; i < 5; i++) {
      saku[i] = saku[i+1];
    }
    saku[4] = ht_calc_saku(saku[3] + 35.0);
  }
  /*
   * saku[0]ʬλʸˤʤäƤޤäˤϡ
   * 򤵤Τܤ­ʤȸơλ򷫤夲ƽ롣
   * κݡ׻saku[0]ˤˤʤäƤʬ䤦ᡢ
   * λ׻롣
   * ʽʬζդǺ󤬤ȵ褦...
   */
  else if (saku[0] > chu[0][0]) {
    for (i = 4; i > 0; i--) {
      saku[i] = saku[i-1];
    }
    saku[0] = ht_calc_saku(saku[0] - 27.0);
  }

  /*
   * ƣ祻å
   * ǣδ֤˺󤬣󤢤ȡǽ롣
   * lap=0:ʿ  lap=1:
   */
  if (saku[4] <= chu[3][0]) {
    lap=1;
  }
  else {
    lap=0;
  }

  /*
   * κ
   * m[i][0] ... ̾1: 2: 3: ....
   * m[i][1] ... ե饰0:ʿ 1:
   * m[i][2] ... jd
   */
  m[0][HT_SAKU_MONTH] = (int)floor(chu[0][1] / 30.0) + 2;
  if (m[0][HT_SAKU_MONTH] > 12) {
    m[0][HT_SAKU_MONTH] -= 12;
  }
  m[0][HT_SAKU_DATE] = (int)floor(saku[0]);
  m[0][HT_SAKU_LEAP] = 0;

  for (i = 1; i < 5; i++) {
    if (lap == 1 && i != 1) {
      if ((int)floor(chu[i-1][0]) <= (int)floor(saku[i-1]) ||
	  (int)floor(chu[i-1][0]) >= (int)floor(saku[i])) {
	m[i-1][HT_SAKU_MONTH] = m[i-2][HT_SAKU_MONTH];
	m[i-1][HT_SAKU_LEAP]  = 1;
	m[i-1][HT_SAKU_DATE]  = (int)floor(saku[i-1]);
	lap = 0;
      }
    }
    m[i][HT_SAKU_MONTH] = m[i-1][HT_SAKU_MONTH] + 1;
    if (m[i][HT_SAKU_MONTH] > 12) {
      m[i][HT_SAKU_MONTH] -= 12;
    }
    m[i][HT_SAKU_DATE] = (int)floor(saku[i]);
    m[i][HT_SAKU_LEAP] = 0;
  }

  /*
   * 󤫤롣
   */
  state = 0;
  for (i = 0; i < 5; i++) {
    if ((int)floor(tm0) < m[i][HT_SAKU_DATE]) {
      state = 1;
      break;
    }
    else if ((int)floor(tm0) == m[i][HT_SAKU_DATE]) {
      state = 2;
      break;
    }
  }
  if (state==0 || state==1) {
    i--;
  }
  kyureki[HT_OLD_LEAP]  = m[i][HT_SAKU_LEAP];
  kyureki[HT_OLD_MONTH] = m[i][HT_SAKU_MONTH];
  kyureki[HT_OLD_DAY]   = (int)floor(tm0) - m[i][HT_SAKU_DATE] + 1;

  /*
   * ǯη׻
   * ʵ10ʾǤĿ礭ˤϡ
   *   ޤǯۤƤʤϤ...
   */
  ht_JD2YMDT(tm0, ymdt);
  kyureki[HT_OLD_YEAR] = ymdt[HT_YMDT_YEAR];
  if (kyureki[HT_OLD_MONTH] > 9 &&
      kyureki[HT_OLD_MONTH] > ymdt[HT_YMDT_MONTH]) {
    kyureki[HT_OLD_YEAR]--;
  }
}

/*
 * 浤λ
 * 
 * ƤӽФ˥åȤѿ
 * tm ........ ׻оݤȤʤʥꥦ
 * chu ....... ͤǼΥݥ󥿡
 * i ......... ͤǼֹ
 *  .... 浤λλβФϤ
 *
 */
static void
ht_calc_chuki(tm, i, chu)
	double	tm;
	int	i;
	double 	chu[][2];
{
  double	tm1, tm2, t;
  double	rm_sun0, rm_sun;
  double	delta_t1, delta_t2, delta_rm;

  /*
   * ʬ򤹤
   */
  tm1 = floor(tm);
  tm2 = tm - tm1;

  /*
   * JST ==> DT =0.0sec ȲꤷƷ׻
   */
  tm2 -= (9.0 / 24.0);

  /*
   * 浤β sun0 
   */
  t = (tm2 + 0.5) / 36525.0;
  t = t + (tm1 - 2451545.0) / 36525.0;
  rm_sun = ht_longitude_sun(t);

  rm_sun0 = 30.0 * floor(rm_sun / 30.0);

  /*
   * ֤׻ˤä浤λ׻
   * ʸ1.0 secˤʤäǤڤ롣
   */
  for (delta_t2 = 1.0; fabs(delta_t1 + delta_t2) > (1.0 / 86400.0); ) {
    /*
     * sun ׻
     */
    t = (tm2 + 0.5) / 36525.0;
    t = t + (tm1 - 2451545.0) / 36525.0;
    rm_sun = ht_longitude_sun(t);

    /*
     * к ˡsun ݦsun0
     */
    delta_rm = rm_sun - rm_sun0 ;

    /*
     * ˤΰϰϡʡ180ˤæˤϡԤ
     */
    if (delta_rm > 180.0) {
      delta_rm -= 360.0;
    }
    else if (delta_rm < -180.0) {
      delta_rm += 360.0;
    }

    /*
     *  t
     * delta_t = delta_rm * 365.2 / 360.0;
     */
    delta_t1  = floor(delta_rm * 365.2 / 360.0);
    delta_t2  = delta_rm * 365.2 / 360.0;
    delta_t2 -= delta_t1;

    /*
     * 
     * tm -= delta_t;
     */
    tm1 = tm1 - delta_t1;
    tm2 = tm2 - delta_t2;
    if (tm2 < 0) {
      tm2 += 1.0;
      tm1 -= 1.0;
    }
  }

  /*
   * ͤκ
   * chu[i][0]:ΤȡDT ==> JST ѴԤͤȤ
   * =0.0sec ȲꤷƷ׻
   * chu[i][1]:
   */
  chu[i][0]  = tm2 + 9.0 / 24.0;
  chu[i][0] += tm1;
  chu[i][1]  = rm_sun0;
}

/*
 * ľʬλ
 *
 * ƤӽФ˥åȤѿ
 * tm ........ ׻оݤȤʤʥꥦ
 * nibun ..... ͤǼΥݥ󥿡
 *  .... ʬλλβФϤ
 * ͤϤäȵˤʤޤ䡣
 */
static void
ht_calc_before_nibun(tm, nibun)
	double	tm;
	double 	nibun[][2];
{
  double	tm1, tm2, t;
  double	rm_sun0, rm_sun;
  double	delta_t1, delta_t2, delta_rm;

  /*
   * ʬ򤹤
   */
  tm1 = floor(tm);
  tm2 = tm - tm1;

  /*
   * JST ==> DT =0.0sec ȲꤷƷ׻
   */
  tm2 -= (9.0 / 24.0);

  /*
   * ľʬβ sun0 
   */
  t = (tm2 + 0.5) / 36525.0;
  t = t + (tm1 - 2451545.0) / 36525.0;
  rm_sun  = ht_longitude_sun(t);
  rm_sun0 = 90 * floor(rm_sun / 90.0);

  /*
   * ֤׻ˤäľʬλ׻
   * ʸ1.0 secˤʤäǤڤ롣
   */
  for (delta_t2 = 1.0; fabs(delta_t1 + delta_t2) > (1.0 / 86400.0); ) {
    /*
     * sun ׻
     */
    t = (tm2 + 0.5) / 36525.0;
    t = t + (tm1 - 2451545.0) / 36525.0;
    rm_sun = ht_longitude_sun(t);

    /*
     * к ˡsun ݦsun0
     */
    delta_rm = rm_sun - rm_sun0;

    /*
     * ˤΰϰϡʡ180ˤæˤϡԤ
     */
    if (delta_rm > 180.0) {
      delta_rm -= 360.0;
    }
    else if (delta_rm < -180.0) {
      delta_rm += 360.0;
    }

    /*
     *  t
     * delta_t = delta_rm * 365.2 / 360.0;
     */
    delta_t1  = floor(delta_rm * 365.2 / 360.0);
    delta_t2  = delta_rm * 365.2 / 360.0;
    delta_t2 -= delta_t1;

    /*
     * 
     * tm -= delta_t;
     */
    tm1 = tm1 - delta_t1;
    tm2 = tm2 - delta_t2;
    if (tm2 < 0) {
      tm2 += 1.0;
      tm1 -= 1.0;
    }
  }

  /*
   * ͤκ
   * nibun[0][0]:ΤȡDT ==> JST ѴԤͤȤ
   * =0.0sec ȲꤷƷ׻
   * nibun[0][1]:
   */
  nibun[0][0]  = tm2 + 9.0 / 24.0;
  nibun[0][0] += tm1;
  nibun[0][1]  = rm_sun0;
}

/*
 * η׻
 * Ϳ줿ľκλJSTˤ
 *
 * ƤӽФ˥åȤѿ
 * tm ........ ׻оݤȤʤʥꥦ
 *  .... λ
 *
 *  ͤȤꥦɽʬäξɽ
 *
 */
static double
ht_calc_saku(tm)
	double	tm;
{
  int		lc;
  double	t, tm1, tm2;
  double	rm_sun, rm_moon;
  double	delta_rm, delta_t1, delta_t2;

  /* 롼ץ󥿤Υå */
  lc = 1;

  /*
   * ʬ򤹤
   */
  tm1 = floor(tm);
  tm2 = tm - tm1;

  /*
   * JST ==> DT =0.0sec ȲꤷƷ׻
   */
  tm2 -= (9.0 / 24.0);

  /*
   * ֤׻ˤäƺλ׻
   * ʸ1.0 secˤʤäǤڤ롣
   */
  for (delta_t2 = 1.0; fabs(delta_t1 + delta_t2) > (1.0 / 86400.0); lc++) {
    /*
     * ۤβЦsun ,βЦmoon ׻
     * t = (tm - 2451548.0 + 0.5)/36525.0;
     */
    t = (tm2 + 0.5) / 36525.0;
    t = t + (tm1 - 2451545.0) / 36525.0;
    rm_sun  = ht_longitude_sun(t);
    rm_moon = ht_longitude_moon(t);

    /*
     * ۤβк
     * ˡmoonݦsun
     */
    delta_rm = rm_moon - rm_sun;

    /*
     * 롼פΣܡlc=1ˤ delta_rm < 0.0 ξˤϰϰϤ
     * 褦
     */
    if (lc == 1 && delta_rm < 0.0) {
      delta_rm = ht_normalization_angle(delta_rm);
    }
    /*
     * ʬζ᤯Ǻ󤬤0 sun 20ˤǡ
     * βЦmoon300 ξˤϡ
     * ˡ 360.0   ȷ׻
     */
    else if (rm_sun >= 0 && rm_sun <= 20 && rm_moon >= 300) {
      delta_rm = ht_normalization_angle(delta_rm);
      delta_rm = 360.0 - delta_rm;
    }
    /*
     * ˤΰϰϡʡ40ˤæˤϡԤ
     */
    else if (fabs(delta_rm) > 40.0) {
      delta_rm = ht_normalization_angle(delta_rm);
    }

    /*
     *  t
     * delta_t = delta_rm * 29.530589 / 360.0;
     */
    delta_t1  = floor(delta_rm * 29.530589 / 360.0);
    delta_t2  = delta_rm * 29.530589 / 360.0;
    delta_t2 -= delta_t1;

    /*
     * 
     * tm -= delta_t;
     */
    tm1 = tm1 - delta_t1;
    tm2 = tm2 - delta_t2;
    if (tm2 < 0.0) {
      tm2 += 1.0;
      tm1 -= 1.0;
    }

    /*
     * 롼ײ15ˤʤä顢 tm  tm-26 Ȥ롣
     */
    if (lc == 15 && fabs(delta_t1 + delta_t2) > (1.0 / 86400.0)) {
      tm1 = floor(tm - 26);
      tm2 = 0;
    }
    /*
     * ͤˤؤ餺ư³ˤϽͤȤ
     * ֤ƶŪ˥롼פȴФư۾ｪλ롣
     */
    else if (lc > 30 && fabs(delta_t1 + delta_t2) > (1.0 / 86400.0)) {
      tm1 = tm;
      tm2 = 0;
      break;
    }
  }

  /*
   * ΤȡDT ==> JST ѴԤͤȤ
   * =0.0sec ȲꤷƷ׻
   */

  return tm2 + tm1 + 9.0 / 24.0;
}

/*
 * ٤ԤʤϰϤ ȡ㣳 ˤ롣
 */
static double
ht_normalization_angle(angle)
	double	angle;
{
  double	angle1;
  double	angle2;

  if (angle < 0.0) {
    angle1  = -angle;
    angle2  = floor(angle1 / 360.0);
    angle1 -= 360.0 * angle2;
    angle1  = 360.0 - angle1;
  }
  else {
    angle1 = floor(angle / 360.0);
    angle1 = angle - 360.0 * angle1;
  }

  return angle1;
}

/*
 * ۤβ sun ׻
 */
static double
ht_longitude_sun(t)
	double	t;
{
  double	th;
  double	ang;

  /*
   * ưη׻
   */
  ang = ht_normalization_angle(  31557.0 * t + 161.0 );
   th =      0.0004 * cos( KXX * ang );
  ang = ht_normalization_angle(  29930.0 * t +  48.0 );
   th = th + 0.0004 * cos( KXX * ang );
  ang = ht_normalization_angle(   2281.0 * t + 221.0 );
   th = th + 0.0005 * cos( KXX * ang );
  ang = ht_normalization_angle(    155.0 * t + 118.0 );
   th = th + 0.0005 * cos( KXX * ang );
  ang = ht_normalization_angle(  33718.0 * t + 316.0 );
   th = th + 0.0006 * cos( KXX * ang );
  ang = ht_normalization_angle(   9038.0 * t +  64.0 );
   th = th + 0.0007 * cos( KXX * ang );
  ang = ht_normalization_angle(   3035.0 * t + 110.0 );
   th = th + 0.0007 * cos( KXX * ang );
  ang = ht_normalization_angle(  65929.0 * t +  45.0 );
   th = th + 0.0007 * cos( KXX * ang );
  ang = ht_normalization_angle(  22519.0 * t + 352.0 );
   th = th + 0.0013 * cos( KXX * ang );
  ang = ht_normalization_angle(  45038.0 * t + 254.0 );
   th = th + 0.0015 * cos( KXX * ang );
  ang = ht_normalization_angle( 445267.0 * t + 208.0 );
   th = th + 0.0018 * cos( KXX * ang );
  ang = ht_normalization_angle(     19.0 * t + 159.0 );
   th = th + 0.0018 * cos( KXX * ang );
  ang = ht_normalization_angle(  32964.0 * t + 158.0 );
   th = th + 0.0020 * cos( KXX * ang );
  ang = ht_normalization_angle(  71998.1 * t + 265.1 );
   th = th + 0.0200 * cos( KXX * ang );
  ang = ht_normalization_angle(  35999.05 * t + 267.52 );
   th = th - 0.0048 * t * cos( KXX * ang ) ;
   th = th + 1.9147     * cos( KXX * ang ) ;

  /*
   * η׻
   */
  ang = ht_normalization_angle( 36000.7695 * t );
  ang = ht_normalization_angle( ang + 280.4659 );
  th  = ht_normalization_angle( th + ang );

  return th;
}

/*
 * β moon ׻
 */
static double
ht_longitude_moon(t)
	double	t;
{
  double	th;
  double	ang;

  /*
   * ưη׻
   */
  ang = ht_normalization_angle( 2322131.0  * t + 191.0  );
   th =      0.0003 * cos( KXX * ang );
  ang = ht_normalization_angle(    4067.0  * t +  70.0  );
   th = th + 0.0003 * cos( KXX * ang );
  ang = ht_normalization_angle(  549197.0  * t + 220.0  );
   th = th + 0.0003 * cos( KXX * ang );
  ang = ht_normalization_angle( 1808933.0  * t +  58.0  );
   th = th + 0.0003 * cos( KXX * ang );
  ang = ht_normalization_angle(  349472.0  * t + 337.0  );
   th = th + 0.0003 * cos( KXX * ang );
  ang = ht_normalization_angle(  381404.0  * t + 354.0  );
   th = th + 0.0003 * cos( KXX * ang );
  ang = ht_normalization_angle(  958465.0  * t + 340.0  );
   th = th + 0.0003 * cos( KXX * ang );
  ang = ht_normalization_angle(   12006.0  * t + 187.0  );
   th = th + 0.0004 * cos( KXX * ang );
  ang = ht_normalization_angle(   39871.0  * t + 223.0  );
   th = th + 0.0004 * cos( KXX * ang );
  ang = ht_normalization_angle(  509131.0  * t + 242.0  );
   th = th + 0.0005 * cos( KXX * ang );
  ang = ht_normalization_angle( 1745069.0  * t +  24.0  );
   th = th + 0.0005 * cos( KXX * ang );
  ang = ht_normalization_angle( 1908795.0  * t +  90.0  );
   th = th + 0.0005 * cos( KXX * ang );
  ang = ht_normalization_angle( 2258267.0  * t + 156.0  );
   th = th + 0.0006 * cos( KXX * ang );
  ang = ht_normalization_angle(  111869.0  * t +  38.0  );
   th = th + 0.0006 * cos( KXX * ang );
  ang = ht_normalization_angle(   27864.0  * t + 127.0  );
   th = th + 0.0007 * cos( KXX * ang );
  ang = ht_normalization_angle(  485333.0  * t + 186.0  );
   th = th + 0.0007 * cos( KXX * ang );
  ang = ht_normalization_angle(  405201.0  * t +  50.0  );
   th = th + 0.0007 * cos( KXX * ang );
  ang = ht_normalization_angle(  790672.0  * t + 114.0  );
   th = th + 0.0007 * cos( KXX * ang );
  ang = ht_normalization_angle( 1403732.0  * t +  98.0  );
   th = th + 0.0008 * cos( KXX * ang );
  ang = ht_normalization_angle(  858602.0  * t + 129.0  );
   th = th + 0.0009 * cos( KXX * ang );
  ang = ht_normalization_angle( 1920802.0  * t + 186.0  );
   th = th + 0.0011 * cos( KXX * ang );
  ang = ht_normalization_angle( 1267871.0  * t + 249.0  );
   th = th + 0.0012 * cos( KXX * ang );
  ang = ht_normalization_angle( 1856938.0  * t + 152.0  );
   th = th + 0.0016 * cos( KXX * ang );
  ang = ht_normalization_angle(  401329.0  * t + 274.0  );
   th = th + 0.0018 * cos( KXX * ang );
  ang = ht_normalization_angle(  341337.0  * t +  16.0  );
   th = th + 0.0021 * cos( KXX * ang );
  ang = ht_normalization_angle(   71998.0  * t +  85.0  );
   th = th + 0.0021 * cos( KXX * ang );
  ang = ht_normalization_angle(  990397.0  * t + 357.0  );
   th = th + 0.0021 * cos( KXX * ang );
  ang = ht_normalization_angle(  818536.0  * t + 151.0  );
   th = th + 0.0022 * cos( KXX * ang );
  ang = ht_normalization_angle(  922466.0  * t + 163.0  );
   th = th + 0.0023 * cos( KXX * ang );
  ang = ht_normalization_angle(   99863.0  * t + 122.0  );
   th = th + 0.0024 * cos( KXX * ang );
  ang = ht_normalization_angle( 1379739.0  * t +  17.0  );
   th = th + 0.0026 * cos( KXX * ang );
  ang = ht_normalization_angle(  918399.0  * t + 182.0  );
   th = th + 0.0027 * cos( KXX * ang );
  ang = ht_normalization_angle(    1934.0  * t + 145.0  );
   th = th + 0.0028 * cos( KXX * ang );
  ang = ht_normalization_angle(  541062.0  * t + 259.0  );
   th = th + 0.0037 * cos( KXX * ang );
  ang = ht_normalization_angle( 1781068.0  * t +  21.0  );
   th = th + 0.0038 * cos( KXX * ang );
  ang = ht_normalization_angle(     133.0  * t +  29.0  );
   th = th + 0.0040 * cos( KXX * ang );
  ang = ht_normalization_angle( 1844932.0  * t +  56.0  );
   th = th + 0.0040 * cos( KXX * ang );
  ang = ht_normalization_angle( 1331734.0  * t + 283.0  );
   th = th + 0.0040 * cos( KXX * ang );
  ang = ht_normalization_angle(  481266.0  * t + 205.0  );
   th = th + 0.0050 * cos( KXX * ang );
  ang = ht_normalization_angle(   31932.0  * t + 107.0  );
   th = th + 0.0052 * cos( KXX * ang );
  ang = ht_normalization_angle(  926533.0  * t + 323.0  );
   th = th + 0.0068 * cos( KXX * ang );
  ang = ht_normalization_angle(  449334.0  * t + 188.0  );
   th = th + 0.0079 * cos( KXX * ang );
  ang = ht_normalization_angle(  826671.0  * t + 111.0  );
   th = th + 0.0085 * cos( KXX * ang );
  ang = ht_normalization_angle( 1431597.0  * t + 315.0  );
   th = th + 0.0100 * cos( KXX * ang );
  ang = ht_normalization_angle( 1303870.0  * t + 246.0  );
   th = th + 0.0107 * cos( KXX * ang );
  ang = ht_normalization_angle(  489205.0  * t + 142.0  );
   th = th + 0.0110 * cos( KXX * ang );
  ang = ht_normalization_angle( 1443603.0  * t +  52.0  );
   th = th + 0.0125 * cos( KXX * ang );
  ang = ht_normalization_angle(   75870.0  * t +  41.0  );
   th = th + 0.0154 * cos( KXX * ang );
  ang = ht_normalization_angle(  513197.9  * t + 222.5  );
   th = th + 0.0304 * cos( KXX * ang );
  ang = ht_normalization_angle(  445267.1  * t +  27.9  );
   th = th + 0.0347 * cos( KXX * ang );
  ang = ht_normalization_angle(  441199.8  * t +  47.4  );
   th = th + 0.0409 * cos( KXX * ang );
  ang = ht_normalization_angle(  854535.2  * t + 148.2  );
   th = th + 0.0458 * cos( KXX * ang );
  ang = ht_normalization_angle( 1367733.1  * t + 280.7  );
   th = th + 0.0533 * cos( KXX * ang );
  ang = ht_normalization_angle(  377336.3  * t +  13.2  );
   th = th + 0.0571 * cos( KXX * ang );
  ang = ht_normalization_angle(   63863.5  * t + 124.2  );
   th = th + 0.0588 * cos( KXX * ang );
  ang = ht_normalization_angle(  966404.0  * t + 276.5  );
   th = th + 0.1144 * cos( KXX * ang );
  ang = ht_normalization_angle(   35999.05 * t +  87.53 );
   th = th + 0.1851 * cos( KXX * ang );
  ang = ht_normalization_angle(  954397.74 * t + 179.93 );
   th = th + 0.2136 * cos( KXX * ang );
  ang = ht_normalization_angle(  890534.22 * t + 145.7  );
   th = th + 0.6583 * cos( KXX * ang );
  ang = ht_normalization_angle(  413335.35 * t +  10.74 );
   th = th + 1.2740 * cos( KXX * ang );
  ang = ht_normalization_angle( 477198.868 * t + 44.963 ); 
   th = th + 6.2888 * cos( KXX * ang );

  /*
   * η׻
   */
  ang = ht_normalization_angle(  481267.8809 * t );
  ang = ht_normalization_angle(  ang + 218.3162 );
  th  = ht_normalization_angle(  th  +  ang );

  return th;
}

/*
 * ǯʬáˤꥦJDˤ׻
 *
 *  δؿǤϡ쥴ꥪˡˤǯΤǤ롣
 *    ʥꥦˡˤǯˤϻѤǤʤ
 */
static double
ht_YMDT2JD(year, month, day, hour, min, sec)
	int	year;
	int	month;
	int	day;
	int	hour;
	int	min;
	int	sec;
{
  double	jd;
  double	t;

  if (month < 3) {
    year  -= 1;
    month += 12;
  }

  jd  = floor(365.25 * year);
  jd += floor(year / 400.0);
  jd -= floor(year / 100.0);
  jd += floor(30.59 * (month - 2));
  jd += 1721088;
  jd += day;

  t  = sec / 3600.0;
  t += min /60.0;
  t += hour;
  t  = t / 24.0;

  jd += t;

  return jd;
}

/*
 * ꥦJDˤǯʬáˤ׻
 *
 * ͤymdt[]
 * ymdt[0] ... ǯ  ymdt[1] ...   ymdt[2] ... 
 * ymdt[3] ...   ymdt[4] ... ʬ  ymdt[5] ... 
 *
 *  δؿǵ᤿ǯϡ쥴ꥪˡˤäɽƤ롣
 *
 */
static void
ht_JD2YMDT(jd, ymdt)
	double	jd;
	int *	ymdt;
{
  int		x0, x1, x2, x3, x4, x5, x6;
  double	tm;

  x0 = (int)floor(jd + 68570.0);
  x1 = (int)floor(x0 / 36524.25);
  x2 = x0 - (int)floor(36524.25 * x1 + 0.75);
  x3 = (int)floor((x2 + 1) / 365.2425);
  x4 = x2 - (int)floor(365.25 * x3) + 31.0;
  x5 = (int)floor((double)x4 / 30.59);
  x6 = (int)floor((double)x5 / 11.0);

  ymdt[HT_YMDT_DAY]   = x4 - (int)floor(30.59 * x5);
  ymdt[HT_YMDT_MONTH] = x5 - 12 * x6 + 2;
  ymdt[HT_YMDT_YEAR]  = 100 * (x1 - 49) + x3 + x6;

  /* 230 */
  if (ymdt[HT_YMDT_MONTH] == 2 && ymdt[HT_YMDT_DAY] > 28) {
    if (ymdt[HT_YMDT_YEAR] % 100 == 0 && ymdt[HT_YMDT_YEAR] % 400 == 0) {
       ymdt[HT_YMDT_DAY] = 29;
    }
    else if (ymdt[HT_YMDT_YEAR] % 4 ==0) {
       ymdt[HT_YMDT_DAY] = 29;
    }
    else{
       ymdt[HT_YMDT_DAY] = 28;
    }
  }

  tm = 86400.0 * (jd - (int)floor(jd));
  ymdt[HT_YMDT_HOUR]   = (int)floor(tm / 3600.0);
  ymdt[HT_YMDT_MINUTE] = (int)floor((tm - 3600.0 * ymdt[HT_YMDT_HOUR]) / 60.0);
  ymdt[HT_YMDT_SECOND] =
	(int)floor(tm - 3600 * ymdt[HT_YMDT_HOUR] - 60 * ymdt[HT_YMDT_MINUTE]);
}

/*
 * 24ᵤɤĴ٤
 * 
 *  .... ׻оݤȤʤǯyear mon day
 *
 *  .... 24ᵤ̾
 *
 *	 WebCalendar2  qreki.pl Υ롼 CǤˤޤ
 *	ɤϡǤޤ 
 *	http://www3.biwako.ne.jp/~nobuaki/webcal/source.html
 */
static const char *
ht_check_24sekki(year, mon, day)
	int	year;
	int	mon;
	int	day;
{
  /* 24ᵤ */
  static const char *	tbl_24sekki[] = {
    "ʬ", "", "", "Ω", "", "", "ƻ", "", "",
    "Ω", "", "Ϫ", "ʬ", "Ϫ", "", "Ω", "", "",
    "߻", "", "紨", "Ω", "", ""
  };
  double	tm, tm1, tm2, t;
  double	rm_sun_today,  rm_sun_tommorow;
  int		rm_sun_today0, rm_sun_tommorow0;

  tm = ht_YMDT2JD(year, mon, day, 0, 0, 0);

  /*
   * ʬ򤹤
   */
  tm1  = floor(tm);
  tm2  = tm - tm1;
  tm2 -= 9.0/24.0;
  t    = (tm2 + 0.5) / 36525.0;
  t   += (tm1 - 2451545.0) / 36525.0;

  /* ۤβ */
  rm_sun_today = ht_longitude_sun(t);

  tm++;
  tm1  = floor(tm);
  tm2  = tm - tm1;
  tm2 -= 9.0/24.0;
  t    = (tm2 + 0.5) / 36525.0;
  t   += (tm1 - 2451545.0) / 36525.0;

  /* ۤβ */
  rm_sun_tommorow = ht_longitude_sun(t);

  rm_sun_today0    = (int)(15.0 * floor(rm_sun_today / 15.0));
  rm_sun_tommorow0 = (int)(15.0 * floor(rm_sun_tommorow / 15.0));
  
  if (rm_sun_today0 != rm_sun_tommorow0) {
    return (tbl_24sekki[rm_sun_tommorow0 / 15]);
  }

  return NULL;
}


/*
 * FUNCTION NAME:	cm_date_get_rokuyou
 * DESCRIPTION:		calculate a rokuyou
 * ARGUMENTS:		year, month, day
 * RETURN VALUE:	successful: rokuyou, otherwise: NULL
 */
static const char *
cm_date_get_rokuyou(year, month, day)
	int	year;
	int	month;
	int	day;
{
  if (cm_date_check(year, month, day)) {
    double	jd;
    int		oldcal[HT_OLD_SIZE];

    jd = ht_YMDT2JD(year, month, day, 0, 0, 0);
    ht_calc_kyureki(jd, oldcal);

    return ht_get_rokuyou(oldcal[HT_OLD_MONTH], oldcal[HT_OLD_DAY]);
  }
  return NULL;
}

/*
 * FUNCTION NAME:	cm_date_get_24sekki
 * DESCRIPTION:		calculate a 24 sekki
 * ARGUMENTS:		year, month, day
 * RETURN VALUE:	successful: 24sekki, otherwise: NULL
 */
static const char *
cm_date_get_24sekki(year, month, day)
	int	year;
	int	month;
	int	day;
{
  if (cm_date_check(year, month, day)) {
    return ht_check_24sekki(year, month, day);
  }
  return NULL;
}

/*
 * FUNCTION NAME:	cm_date_get_old_calendar
 * DESCRIPTION:		calculate an old_calendar
 * ARGUMENTS:		year, month, day, retval array
 * RETURN VALUE:	successful: 0, otherwise: -1
 */
static int
cm_date_get_old_calendar(year, month, day, retval)
	int	year;
	int	month;
	int	day;
	int *	retval;
{
  if (cm_date_check(year, month, day)) {
    double	jd;
    int		oldcal[HT_OLD_SIZE];

    jd = ht_YMDT2JD(year, month, day, 0, 0, 0);
    ht_calc_kyureki(jd, oldcal);

    retval[CC_OLDCAL_YEAR]  = oldcal[HT_OLD_YEAR];
    retval[CC_OLDCAL_LEAP]  = oldcal[HT_OLD_LEAP];
    retval[CC_OLDCAL_MONTH] = oldcal[HT_OLD_MONTH];
    retval[CC_OLDCAL_DAY]   = oldcal[HT_OLD_DAY];

    return 0;
  }

  return -1;
}

/*****************************************************************************/

static const cmDate	date_funcs = {
  cm_date_get_today,
  cm_date_get_year_of_today,
  cm_date_get_month_of_today,
  cm_date_get_day_of_today,
  cm_date_get_wday_of_today,
  cm_date_is_festival,
  cm_date_get_festival_name,
  cm_date_is_holiday_in_lieu,
  cm_date_get_day_of_the_week,
  cm_date_get_time,
  cm_date_check,
  cm_date_get_rokuyou,
  cm_date_get_24sekki,
  cm_date_get_old_calendar
};

static void
cm_date_register_hooks(void)
{
  cc_hook_module_date(&date_funcs);
}

static ccModule	mod_date = {
  CC_MODULE_MAGIC_NUMBER_MAJOR,		/* API version */
  "mod_date",				/* module name */
  "Cockatrice: date module 1.0.3",	/* description such as version etc.. */
  NULL,					/* module handle */
  (void *)cm_date_init,			/* called when it's loaded */
  (void *)NULL,				/* called when it's unloaded */
  (void *)cm_date_register_hooks
};

ccModule *
ccModuleInfo(void)
{
  return &mod_date;
}

/*****************************************************************************/

#if	DATE_DEBUG
int
main(argc, argv)
	int	argc;
	char *	argv[];
{
  int	year;
  int	month;
  int	day;

  if (argc != 4) {
    fprintf(stderr, "usage: %s year month day\n", argv[0]);
    exit(64);
  }

  year	= (int)strtol(argv[1], (char **)NULL, 10);
  month = (int)strtol(argv[2], (char **)NULL, 10);
  day	= (int)strtol(argv[3], (char **)NULL, 10);
  if (cm_date_is_festival(year, month, day)) {
    fprintf(stdout, "%d/%02d/%02d is festival\n", year, month, day);
  }
  else if (cm_date_is_holiday_in_lieu(year, month, day)) {
    fprintf(stdout, "%d/%02d/%02d is holiday\n", year, month, day);
  }
  else {
    fprintf(stdout, "%d/%02d/%02d is ordinary day\n", year, month, day);
  }

  return 0;
}
#endif	/* DATE_DEBUG */
